import { FeatureStateValueInterceptor, InterceptorValueMatch } from './interceptors';
import { FeatureStateHolder } from './feature_state';
import { AnalyticsCollector } from './analytics';
import { RolloutStrategy, SSEResultState } from './models';
import { ClientContext } from './client_context';
import { Applied, ApplyFeature } from './strategy_matcher';
import { InternalFeatureRepository } from './internal_feature_repository';
import { PostLoadNewFeatureStateAvailableListener, Readyness, ReadynessListener } from './featurehub_repository';
export declare class ClientFeatureRepository implements InternalFeatureRepository {
    private hasReceivedInitialState;
    private features;
    private analyticsCollectors;
    private readynessState;
    private readynessListeners;
    private _catchAndReleaseMode;
    private _catchReleaseStates;
    private _newFeatureStateAvailableListeners;
    private _matchers;
    private readonly _applyFeature;
    constructor(applyFeature?: ApplyFeature);
    apply(strategies: Array<RolloutStrategy>, key: string, featureValueId: string, context: ClientContext): Applied;
    get readyness(): Readyness;
    notify(state: SSEResultState, data: any): void;
    addValueInterceptor(matcher: FeatureStateValueInterceptor): void;
    valueInterceptorMatched(key: string): InterceptorValueMatch;
    addPostLoadNewFeatureStateAvailableListener(listener: PostLoadNewFeatureStateAvailableListener): void;
    addReadynessListener(listener: ReadynessListener): void;
    notReady(): void;
    broadcastReadynessState(): Promise<void>;
    addAnalyticCollector(collector: AnalyticsCollector): void;
    simpleFeatures(): Map<string, string | undefined>;
    logAnalyticsEvent(action: string, other?: Map<string, string>, ctx?: ClientContext): Promise<void>;
    hasFeature(key: string): FeatureStateHolder;
    feature(key: string): FeatureStateHolder;
    getFeatureState(key: string): FeatureStateHolder;
    get catchAndReleaseMode(): boolean;
    set catchAndReleaseMode(value: boolean);
    release(disableCatchAndRelease?: boolean): Promise<void>;
    getFlag(key: string): boolean | undefined;
    getString(key: string): string | undefined;
    getJson(key: string): string | undefined;
    getNumber(key: string): number | undefined;
    isSet(key: string): boolean;
    private _catchUpdatedFeatures;
    private triggerNewStateAvailable;
    private featureUpdate;
    private deleteFeature;
}
